addpath('LIB')

%{
AUTHOR:        Nikolai Yurttagl
DATE:          21-02-2019

This script calculates a distribution of tunnel conductances for a given
set of temperatures in the presence of random offset charges.

%}

% Island Capacitance
C0 = 669.8e-15;

% Capacitance to Ground
CG = 1.8e-15;

% Tunnel resistance
R0 = 35e3;

% Number of islands in series
N_SERIES = 35;

% Number of CBT linear arrays in parallel
N_PARALLEL = 5;

% Asymptotic tunnel conductance
Gt = 1/((N_SERIES+1)*R0);

% Number of random offset charges taken into account for a single linear
% array
REPEATS = 1000;

% Evaluated temperatures at which the zero bias tunnel conductance is
% calculated. The bias offset from zero to calculate the differential 
% conductance was determined using the master equation tunneling model.

%{ 
IMPORTANT: 
The number of steps in every Markov chain (STEPS_T) plays an important role
because it takes at least 1e6 steps in order to not increase the standard deviation
of the conductance distribution (G_DIST) by the non-convergence of the Markov chain.
The standard deviation of G_DIST should be only determined by the offset charges
on the islands. Check the variation of the current in the high temperature
limit to find out how much steps are needed for convergence if needed.
%} 

T       = [1.0e-1 5.0e-2 3.0e-2 1.75e-2 1.0e-2 7.5e-3 5.0e-3 3.5e-3 2.5e-3 2.0e-3 1.5e-3 1.25e-3 1.0e-3 9.0e-4 8.5e-4 7.5e-4 7.0e-4 6.5e-4 6.0e-4 5.5e-4 5.0e-4 4.5e-4 4.0e-4 3.5e-4 3.25e-4 3.0e-4 2.75e-4 2.5e-4 2.3e-4 2.0e-4 1.5e-4 1.0e-4 7.5e-5 5.0e-5 3.5e-5 2.5e-5 1.5e-5 1.0e-5];
STEPS_T = [1.0e+6 1.0e+6 1.0e+6 1.0e+6  1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6  1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6  1.0e+6 1.0e+6  1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6 1.0e+6];
VOLTAGE = [0.0e-0 0.0e-0 0.0e-0 0.0e-0  0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0  0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0  0.0e-0 0.0e-0  0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0 0.0e-0];
DV      = [1.5e-4 5.0e-5 4.0e-5 2.5e-5  5.0e-6 5.0e-6 2.5e-6 2.0e-6 1.5e-6 1.0e-6 5.0e-7 5.0e-7  5.0e-7 3.0e-7 3.0e-7 3.0e-7 3.0e-7 3.0e-7 3.0e-7 3.0e-7 3.0e-7 2.0e-7 2.0e-7 2.0e-7 2.0e-7  2.0e-7 2.0e-7  2.0e-7 2.0e-7 2.0e-7 2.0e-7 1.0e-7 1.0e-7 1.0e-7 1.0e-7 1.0e-7 1.0e-7 1.0e-7];

% We calculate the tunnel conductance from the given bias and the current
% which is calculated with a Markov model

dG  = zeros(REPEATS,length(T));
dI  = zeros(2*REPEATS,length(T));
dV  = zeros(2*REPEATS,length(T));
POS = 1;

% Number of iterations for calculating the tunnel conductance from
% a full (N+1)xN_PARALLEL array by averaging over N_PARALLEL samples
% of the conductance distribution of a linear 1x(N+1) array.

NUMBER_OF_SAMPLES = 1e6;

% conductance distribution of the CBT at different temperatures 
% as the end result

G_DIST = zeros(NUMBER_OF_SAMPLES,length(T));

for i=1:length(T)

disp(['Temperature = ' num2str(T(i).*1e3) ' mK'])    
    
for j=1:REPEATS    
    
% Create a random offset charge configuration    
    
dQ_LIM_1 = -0.5;
dQ_LIM_2 = 0.5;
OFFSET_CHARGE = (dQ_LIM_2-dQ_LIM_1).*rand(N_SERIES,1) + dQ_LIM_1;
       
% Calculate the current at the given input voltage using the Markov Model

V = [VOLTAGE(i)-DV(i) VOLTAGE(i)+DV(i)]; 
I = ISLAND_ARRAY_PHI_IV(N_SERIES,C0,CG,R0,T(i),V,OFFSET_CHARGE,(STEPS_T(i)).*ones(length(V),1));

% Calculate the differential conductance and store the current and voltage

X = [ones(length(V),1) V'];
FIT = X\(I');

dG(j,i) = FIT(2);

dI(POS:POS+1,i) = I;
dV(POS:POS+1,i) = V;

POS = POS+2;

end

% Now we loop over the conductance distribution of the linear array and
% pick N_PARALLEL random samples of which we calculate the average.

for k=1:NUMBER_OF_SAMPLES
    
r = randi([1 REPEATS],1,N_PARALLEL);

G_DIST(k,i) = mean(dG(r,i)./Gt);

end

POS = 1;

end

% Save the result

save DATA/RANDOM_OFFSET_CHARGE_ARRAY_CONDUCTANCE.mat dI dV dG G_DIST Gt C0 CG R0 T VOLTAGE STEPS_T N_SERIES N_PARALLEL
