function [CURRENT] = ISLAND_ARRAY_PHI_IV(N0,...
                                         C0,...
                                         CG,...
                                         R0,...
                                         T0,...
                                         VOLTAGE,...
                                         OFFSET_CHARGE,...
                                         STEPS)

%{
FUNCTION NAME: ISLAND_ARRAY_PHI
AUTHOR:        Nikolai Yurttagl
DATE:          21-02-2019

The function ISLAND_ARRAY_PHI models an one-dimensional array of tunnel 
junctions enclosing N0 islands. Each island has
a defined coupling capacitance to its neighbours which is given by C0.
Additionally each island couples to Ground with a ground capacitance CG.
The charge accumulated on an island at a given temperature and bias then
is a sum of the charge carriers tunneled onto- or from that specific island
as well as gate charge induced by the voltage to ground and an offset
charge being the trapped charge on each island which cannot be removed
by integer charge tunneling. STEPS is the number of elements in the Markov
chain, STEPS should be at least 1e6.                                         
%}
                                                                        
[dataFilePath,...
 dataFileName, ~] = fileparts('INIT.ini');
metaDataFile      = fullfile(dataFilePath, [dataFileName '.ini']);
metadata          = INI2STRUCT(metaDataFile);
e                 = str2double(metadata.constants.elem_charge);
kb                = str2double(metadata.constants.boltz_const);

% The number of tunnel junctions in series for an array of N0 islands.

n=N0+1;

% The Matrix M is used to calculate the electrostatic potential of each 
% island.

DIAGONAL     =  (2+CG/C0).*ones(N0,1);
OFF_DIAGONAL = -1.*ones(N0-1,1);

M = diag(DIAGONAL)...
  + diag(OFF_DIAGONAL,1)...
  + diag(OFF_DIAGONAL,-1);

% The Operator to calculate the free energy change of a single charge
% tunneling event for a specific island.

DELTAPHI = zeros(N0-1,N0);

for i=1:(N0-1)
    
DELTAPHI(i,i:(i+1)) = [e/2 -e/2]; 
    
end

DELTAPHI = [[-e/2 zeros(1,n-2)]; DELTAPHI; [zeros(1,n-2) e/2]];

% The dQ matrix contains all possible charge number changes which
% can occur by single charge tunneling events.

dQ = diag(ones(N0+1,1))...
       + diag(-1.*ones(N0,1),-1);
dQ(N0+2,N0+1)=-1;
dQ = [dQ -1.*dQ];

% The current return value

CURRENT = zeros(1,length(VOLTAGE));

% In the main loop we create a Markov Chain

for k=1:length(VOLTAGE)
    disp(['V = ' num2str(VOLTAGE(k)) ' V'])  

% For each bias the device is reinitialized with zero charge on each island,
% the offset charge which is given as an input parameter is then added to
% each island. We work with a charge distribution divided by C0 which
% allows to add the electrostatic potential of the leads in a proper way.

N = zeros(N0,1); 
N(1)  = N(1)  - VOLTAGE(k)/2;
N(N0) = N(N0) + VOLTAGE(k)/2;

N = N + e*OFFSET_CHARGE/C0;

% Calculate the inital electrostatic energy of the device

phi = M\N;

% Set the clock to zero as well as the transported charge.

Delta_Q = 0;
Delta_t = 0;

% In the main loop we create a Markov Chain

for i=1:STEPS (k)  

T = repmat(N,1,2*n)+dQ(2:(N0+1),:).*e/C0;
 
PHI = [phi M\T];

DELTAF = DELTAPHI*PHI;

dF = [(diag(DELTAF(:,2:n+1))+DELTAF(:,1)); ...
     -(diag(DELTAF(:,(n+2):(2*n+1)))+DELTAF(:,1))] ...
     +[-e*VOLTAGE(k)/2;...
        zeros(N0-1,1); ...
       -e*VOLTAGE(k)/2; ...
        e*VOLTAGE(k)/2; ...
        zeros(N0-1,1); ...
        e*VOLTAGE(k)/2];

% Calculate the tunneling rate for each possible single charge tunneling
% event
    
GAMMA = (1/(e^2*R0)).*((-dF+e*eps)./(1-exp((dF+e*eps)./(kb*T0))));

% The timestep of the corresponding markov chain element is calculated by
% normalizing the tunneling rates.

dt = 1/sum(GAMMA);

% The tunneling probabilities

P = GAMMA.*dt;

% We pick a single charge tunneling event. The process is weighted with
% the probabilities P.

s = randsample(1:length(P),1,true,P);

% Change the charge configuration according to the chosen tunneling event

N = N + dQ(2:N0+1,s)*e/C0;

% Update the transported charge and the passed time.

Delta_Q = Delta_Q + (e/n)*(sum(GAMMA(1:n)-GAMMA((n+1):(2*n))))/(sum(GAMMA));
Delta_t = Delta_t + dt;

% The electrostatic potential corresponding to the randomly selected
% tunneling event is taken as the inital electrostatic potential of the
% next step in the Markov chain.

phi = PHI(:,s+1);

end

% Store the calculated current for the given input voltage.

CURRENT(k) = Delta_Q/Delta_t;

end

% FUNCTION ENDS HERE

end